"use strict";

function _toConsumableArray(arr) {
    return (
        _arrayWithoutHoles(arr) ||
        _iterableToArray(arr) ||
        _unsupportedIterableToArray(arr) ||
        _nonIterableSpread()
    );
}

function _nonIterableSpread() {
    throw new TypeError(
        "Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."
    );
}

function _unsupportedIterableToArray(o, minLen) {
    if (!o) return;
    if (typeof o === "string") return _arrayLikeToArray(o, minLen);
    var n = Object.prototype.toString.call(o).slice(8, -1);
    if (n === "Object" && o.constructor) n = o.constructor.name;
    if (n === "Map" || n === "Set") return Array.from(o);
    if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n))
        return _arrayLikeToArray(o, minLen);
}

function _iterableToArray(iter) {
    if (typeof Symbol !== "undefined" && Symbol.iterator in Object(iter))
        return Array.from(iter);
}

function _arrayWithoutHoles(arr) {
    if (Array.isArray(arr)) return _arrayLikeToArray(arr);
}

function _arrayLikeToArray(arr, len) {
    if (len == null || len > arr.length) len = arr.length;
    for (var i = 0, arr2 = new Array(len); i < len; i++) {
        arr2[i] = arr[i];
    }
    return arr2;
}

var os = require("os");

var path = require("path");

var fse = require("fs-extra");

var fs = require("fs");

var axios = require("axios");

var _require = require("child_process"),
    spawn = _require.spawn;

var _require2 = require("enquirer"),
    prompt = _require2.prompt,
    Select = _require2.Select,
    Toggle = _require2.Toggle;

var _require3 = require("console"),
    Console = _require3.Console;

var marked = require("marked");

var TerminalRenderer = require("marked-terminal");

marked.setOptions({
    renderer: new TerminalRenderer()
});

function Core(target) {
    var configuration = fse.readJsonSync(target, {
        throws: false
    });
    var ptr = null;
    var ready = true;
    this.configuration = configuration;
    this.target = target;
    this.api = "https://tanker.xarlike.info/api/";
    this.timeout = 1000;

    if (
        configuration &&
        configuration.api &&
        configuration.timeout &&
        configuration.token
    ) {
        var baseURL = configuration.api,
            timeout = configuration.timeout,
            token = configuration.token;
        var params = {
            baseURL: this.api,
            timeout: timeout,
            headers: {
                Authorization: "Bearer ".concat(token)
            }
        };
        ptr = axios.create(params);
    }

    this.axios = ptr;
}

Core.prototype.before = function (message) {
    var axios = this.axios;

    if (axios === null) {
        throw new Error(message);
    }
};

Core.prototype.error = function (message) {
    console.error(message);
};

Core.prototype.debug = async function () {
    var configuration = this.configuration;
    console.log("configuration", configuration);
};

Core.prototype.random = function (len) {
    return _toConsumableArray(Array(12))
        .map(function (i) {
            return (~~(Math.random() * len)).toString(len);
        })
        .join("");
};

Core.prototype.configure = async function () {
    var api = this.api,
        timeout = this.timeout,
        target = this.target;

    try {
        var token = await prompt({
            type: "input",
            name: "token",
            message: "What is your token ?"
        });
        this.write(target, token, {
            api: api,
            timeout: timeout
        });
        console.log(
            'Token added, the next step is to run this command "tanker-cli use"'
        );
    } catch (e) { }
};

Core.prototype.write = async function (target, configuration, extension) {
    try {
        var state = Object.assign({}, configuration, extension);
        fse.writeJsonSync(target, state);
    } catch (e) {
        console.error(e.message);
    }
};

Core.prototype.reset = async function () {
    var target = this.target;

    try {
        this.before("There is no configuration to delete.");

        var _prompt = new Toggle({
            message: "Do you can to remove your access token ?",
            enabled: "Yes",
            disabled: "No"
        });

        _prompt
            .run()
            .then(function (answer) {
                answer && fse.writeJsonSync(target, {});
            })
            .catch(function (e) { });
    } catch (e) {
        this.error(e.message);
    }
};

Core.prototype.use = async function () {
    var configuration = this.configuration,
        axios = this.axios,
        target = this.target;
    var message = 'Need to run this command "tanker-cli configure" first';

    try {
        this.before(message);

        var _await$axios$get = await axios.get("/codes"),
            data = _await$axios$get.data;

        var choices = data.map(function (el) {
            return {
                name: el.title,
                value: el.id
            };
        });

        if (choices.length === 0) {
            return console.error("Nothing found");
        }

        var _prompt2 = new Select({
            message: "Which script do you want to select ?",
            choices: choices,
            result: function result(names) {
                return this.map(names);
            }
        });

        var res = await _prompt2.run();
        var keys = Object.keys(res);

        for (var i = 0; i < keys.length; i++) {
            var key = keys[i];
            var codeId = res[key];

            var _await$axios$get2 = await axios.get("/codes/".concat(codeId)),
                _data = _await$axios$get2.data;

            this.write(target, configuration, {
                payload: _data
            });
        }
    } catch (e) {
        var _e$response;

        if (
            (e === null || e === void 0
                ? void 0
                : (_e$response = e.response) === null || _e$response === void 0
                    ? void 0
                    : _e$response.status) === 401
        ) {
            console.error("401 - Unauthorized");
            return console.error(message);
        }

        console.log(e.message);
    }
};

Core.prototype.run = async function (_ref) {
    var bin = _ref.bin;
    var configuration = this.configuration;
    var once = this.random(36);
    var fileName = path.join(os.tmpdir(), "payload-".concat(once, ".sh"));

    try {
        var _configuration$payloa;

        if (
            configuration === null || configuration === void 0
                ? void 0
                : (_configuration$payloa = configuration.payload) === null ||
                    _configuration$payloa === void 0
                    ? void 0
                    : _configuration$payloa.content
        ) {
            fse.writeFileSync(fileName, configuration.payload.content, {
                mode: 320
            });
        } else {
            return this.error('Need to run this command "tanker-cli use" first');
        }

        var args = _toConsumableArray(process.argv);

        while (args.shift() !== "run") { }

        args.unshift(fileName);
        var bat = spawn(bin, args);
        bat.stdout.on("data", function (data) {
            process.stdout.write(data.toString());
        });
        bat.stderr.on("data", function (data) {
            process.stderr.write(data.toString());
        });
    } catch (e) {
        return console.error(e);
    }
};

Core.prototype.display = async function () {
    var _configuration$payloa2;

    var configuration = this.configuration;

    if (
        configuration === null || configuration === void 0
            ? void 0
            : (_configuration$payloa2 = configuration.payload) === null ||
                _configuration$payloa2 === void 0
                ? void 0
                : _configuration$payloa2.content
    ) {
        return process.stdout.write(configuration.payload.content);
    } else {
        return this.error('Need to run this command "tanker-cli use" first');
    }
};

Core.prototype.status = async function () {
    var axios = this.axios,
        configuration = this.configuration;
    var message = 'Need to run this command "tanker-cli configure" first';

    try {
        this.before(message);

        var _await$axios$get3 = await axios.get("/user"),
            data = _await$axios$get3.data;

        console.log("200 - Access Granted");
        console.log("----------");
        console.log("name: ".concat(data.name));
        console.log("email: ".concat(data.email));
        console.log("----------");

        if (configuration && configuration.payload) {
            var payload = configuration.payload;
            console.log("Title:");
            console.log(payload.title);
            console.log("Summary:");
            console.log(payload.summary);
            console.log("Description:");
            console.log(
                payload.description === null ? null : marked(payload.description)
            );
        }
    } catch (e) {
        var _e$response2;

        if (
            (e === null || e === void 0
                ? void 0
                : (_e$response2 = e.response) === null || _e$response2 === void 0
                    ? void 0
                    : _e$response2.status) === 401
        ) {
            this.error("401 - Unauthorized");
            return this.error(message);
        }

        console.log(e.message);
    }
};

Core.prototype.add = async function (title, env) {
    var axios = this.axios,
        api = this.api,
        configuration = this.configuration,
        target = this.target;
    var message = 'Need to run this command "tanker-cli configure" first';

    try {
        this.before(message);
        var file = env.file,
            description = env.description,
            summary = env.summary,
            isPrivate = env.private;
        var isPublic = isPrivate === true ? false : true;
        var params = {
            title: title,
            summary: summary,
            public: isPublic
        };

        if (description) {
            params.description = fse.readFileSync(description, "utf8");
        }

        if (file) {
            params.content = fse.readFileSync(file, "utf8");
        }

        var _await$axios$post = await axios.post("/codes", params),
            data = _await$axios$post.data;

        this.write(target, configuration, {
            payload: data
        });
        console.log("Created !");
    } catch (e) {
        var _e$response3, _e$response4;

        if (
            (e === null || e === void 0
                ? void 0
                : (_e$response3 = e.response) === null || _e$response3 === void 0
                    ? void 0
                    : _e$response3.status) === 401
        ) {
            this.error("401 - Unauthorized");
            return this.error(message);
        }

        if (
            (e === null || e === void 0
                ? void 0
                : (_e$response4 = e.response) === null || _e$response4 === void 0
                    ? void 0
                    : _e$response4.status) === 403
        ) {
            return console.error("Go visit https://tanker.xarlike.info/billing");
        }

        console.log(e.message);
    }
};

Core.prototype.update = async function (status, env) {
    var axios = this.axios,
        api = this.api,
        configuration = this.configuration,
        target = this.target;
    var message = 'Need to run this command "tanker-cli configure" first';

    try {
        var _configuration$payloa3;

        this.before(message);
        var id = null;

        if (
            configuration === null || configuration === void 0
                ? void 0
                : (_configuration$payloa3 = configuration.payload) === null ||
                    _configuration$payloa3 === void 0
                    ? void 0
                    : _configuration$payloa3.id
        ) {
            id = configuration.payload.id;
        } else {
            return this.error('Need to run this command "tanker-cli use" first');
        }

        var title = env.title,
            file = env.file,
            description = env.description,
            summary = env.summary;
        var params = {};

        if (status === "public") {
            params.public = true;
        } else if (status === "private") {
            params.public = false;
        }

        if (title !== null) {
            params.title = title;
        }

        if (summary !== null) {
            params.summary = summary;
        }

        if (description !== null) {
            params.description = fs.readFileSync(description, "utf8");
        }

        if (file !== null) {
            params.content = fs.readFileSync(file, "utf8");
        }

        var _await$axios$patch = await axios.patch("/codes/".concat(id), params),
            data = _await$axios$patch.data;

        this.write(target, configuration, {
            payload: data
        });
        console.log("Updated !");
    } catch (e) {
        var _e$response5, _e$response6;

        if (
            (e === null || e === void 0
                ? void 0
                : (_e$response5 = e.response) === null || _e$response5 === void 0
                    ? void 0
                    : _e$response5.status) === 401
        ) {
            this.error("401 - Unauthorized");
            return this.error(message);
        }

        if (
            (e === null || e === void 0
                ? void 0
                : (_e$response6 = e.response) === null || _e$response6 === void 0
                    ? void 0
                    : _e$response6.status) === 403
        ) {
            return console.error("Go visit https://tanker.xarlike.info/billing");
        }

        console.log(e.message);
    }
};

var generate = function generate() {
    var filename = ".tanker";
    var target = path.join(process.env.HOME, filename);
    var core = new Core(target);
    core.Core = Core;
    return core;
};

module.exports = generate();




// const os = require('os');
// const path = require('path');
// const fse = require('fs-extra');
// const fs = require('fs');
// const axios = require('axios');
// const { spawn } = require('child_process');
// const { prompt, Select, Toggle } = require('enquirer');
// const { Console } = require('console');
// const marked = require('marked');
// const TerminalRenderer = require('marked-terminal');

// marked.setOptions({
//     renderer: new TerminalRenderer()
// });

// function Core(target) {
//     const configuration = fse.readJsonSync(target, { throws: false });

//     let ptr = null;
//     let ready = true;

//     this.configuration = configuration;
//     this.target = target;
//     this.api = `https://tanker.xarlike.info/api/`;
//     this.timeout = 1000;

//     if (configuration && configuration.api && configuration.timeout && configuration.token) {
//         const { api: baseURL, timeout, token } = configuration;
//         const params = {
//             baseURL: this.api,
//             timeout,
//             headers: { 'Authorization': `Bearer ${token}` }
//         };
//         ptr = axios.create(params);
//     }
//     this.axios = ptr;
// }

// Core.prototype.before = function (message) {
//     const { axios } = this;
//     if (axios === null) {
//         throw new Error(message);
//     }
// };

// Core.prototype.error = function (message) {
//     console.error(message);
// };

// Core.prototype.debug = async function () {
//     const { configuration } = this;
//     console.log("configuration", configuration);
// };

// Core.prototype.random = function (len) {
//     return [...Array(12)].map(i => (~~(Math.random() * len)).toString(len)).join('');
// };

// Core.prototype.configure = async function () {
//     const { api, timeout, target } = this;

//     try {
//         const token = await prompt({
//             type: 'input',
//             name: 'token',
//             message: 'What is your token ?'
//         });

//         this.write(target, token, { api, timeout });
//         console.log('Token added, the next step is to run this command "tanker-cli use"');
//     } catch (e) { }
// };

// Core.prototype.write = async function (target, configuration, extension) {
//     try {
//         const state = Object.assign({}, configuration, extension);
//         fse.writeJsonSync(target, state);
//     } catch (e) {
//         console.error(e.message);
//     }
// };

// Core.prototype.reset = async function () {
//     const { target } = this;

//     try {
//         this.before('There is no configuration to delete.');
//         const prompt = new Toggle({
//             message: 'Do you can to remove your access token ?',
//             enabled: 'Yes',
//             disabled: 'No'
//         });
//         prompt.run()
//             .then(answer => {
//                 answer && fse.writeJsonSync(target, {})
//             })
//             .catch((e) => { });
//     } catch (e) {
//         this.error(e.message);
//     }
// };

// Core.prototype.use = async function () {
//     const { configuration, axios, target } = this;
//     const message = 'Need to run this command "tanker-cli configure" first';
//     try {
//         this.before(message);

//         const { data } = await axios.get("/codes");

//         const choices = data.map((el) => {
//             return { name: el.title, value: el.id };
//         });

//         if (choices.length === 0) {
//             return console.error('Nothing found');
//         }

//         const prompt = new Select({
//             message: 'Which script do you want to select ?',
//             choices,
//             result(names) {
//                 return this.map(names);
//             }
//         });

//         const res = await prompt.run()
//         const keys = Object.keys(res);
//         for (let i = 0; i < keys.length; i++) {
//             const key = keys[i];
//             const codeId = res[key];
//             const { data } = await axios.get(`/codes/${codeId}`);
//             this.write(target, configuration, { payload: data });
//         }
//     } catch (e) {
//         if (e?.response?.status === 401) {
//             console.error("401 - Unauthorized");
//             return console.error(message);
//         }
//         console.log(e.message);
//     }
// };

// Core.prototype.run = async function ({ bin }) {
//     const { configuration } = this;

//     const once = this.random(36);
//     const fileName = path.join(os.tmpdir(), `payload-${once}.sh`);

//     try {
//         if (configuration?.payload?.content) {
//             fse.writeFileSync(fileName, configuration.payload.content, { mode: 0o500 });
//         } else {
//             return this.error('Need to run this command "tanker-cli use" first');
//         }

//         const args = [...process.argv];
//         while (args.shift() !== "run") { }
//         args.unshift(fileName);
//         const bat = spawn(bin, args);
//         bat.stdout.on('data', (data) => {
//             process.stdout.write(data.toString());
//         });
//         bat.stderr.on('data', (data) => {
//             process.stderr.write(data.toString());
//         });
//     } catch (e) {
//         return console.error(e);
//     }
// };

// Core.prototype.display = async function () {
//     const { configuration } = this;
//     if (configuration?.payload?.content) {
//         return process.stdout.write(configuration.payload.content);
//     } else {
//         return this.error('Need to run this command "tanker-cli use" first');
//     }
// };

// Core.prototype.status = async function () {
//     const { axios, configuration } = this;

//     const message = 'Need to run this command "tanker-cli configure" first';
//     try {
//         this.before(message);

//         const { data } = await axios.get(`/user`);
//         console.log("200 - Access Granted");
//         console.log('----------');
//         console.log(`name: ${data.name}`);
//         console.log(`email: ${data.email}`);
//         console.log('----------');
//         if (configuration && configuration.payload) {
//             const { payload } = configuration;
//             console.log('Title:');
//             console.log(payload.title);
//             console.log('Summary:');
//             console.log(payload.summary);
//             console.log('Description:');
//             console.log(payload.description === null ? null : marked(payload.description));
//         }
//     } catch (e) {
//         if (e?.response?.status === 401) {
//             this.error("401 - Unauthorized");
//             return this.error(message);
//         }
//         console.log(e.message);
//     }
// };

// Core.prototype.add = async function (title, env) {
//     const { axios, api, configuration, target } = this;

//     const message = 'Need to run this command "tanker-cli configure" first';
//     try {
//         this.before(message);

//         const { file, description, summary, private: isPrivate } = env;
//         const isPublic = (isPrivate === true) ? false : true;
//         const params = {
//             title,
//             summary,
//             public: isPublic
//         };
//         if (description) {
//             params.description = fse.readFileSync(description, "utf8");
//         }
//         if (file) {
//             params.content = fse.readFileSync(file, "utf8");
//         }
//         const { data } = await axios.post('/codes', params);
//         this.write(target, configuration, { payload: data });
//         console.log('Created !');
//     } catch (e) {
//         if (e?.response?.status === 401) {
//             this.error("401 - Unauthorized");
//             return this.error(message);
//         }
//         if (e?.response?.status === 403) {
//             return console.error("Go visit https://tanker.xarlike.info/billing");
//         }
//         console.log(e.message);
//     }
// };

// Core.prototype.update = async function (status, env) {
//     const { axios, api, configuration, target } = this;

//     const message = 'Need to run this command "tanker-cli configure" first';
//     try {
//         this.before(message);

//         let id = null;
//         if (configuration?.payload?.id) {
//             id = configuration.payload.id;
//         } else {
//             return this.error('Need to run this command "tanker-cli use" first');
//         }

//         const { title, file, description, summary } = env;
//         const params = {};
//         if (status === "public") {
//             params.public = true;
//         } else if (status === "private") {
//             params.public = false;
//         }
//         if (title !== null) {
//             params.title = title;
//         }
//         if (summary !== null) {
//             params.summary = summary;
//         }
//         if (description !== null) {
//             params.description = fs.readFileSync(description, "utf8");
//         }
//         if (file !== null) {
//             params.content = fs.readFileSync(file, "utf8");
//         }
//         const { data } = await axios.patch(`/codes/${id}`, params);
//         this.write(target, configuration, { payload: data });
//         console.log('Updated !');
//     } catch (e) {
//         if (e?.response?.status === 401) {
//             this.error("401 - Unauthorized");
//             return this.error(message);
//         }
//         if (e?.response?.status === 403) {
//             return console.error("Go visit https://tanker.xarlike.info/billing");
//         }
//         console.log(e.message);
//     }
// };

// const generate = () => {
//     const filename = '.tanker';
//     const target = path.join(process.env.HOME, filename);

//     const core = new Core(target);
//     core.Core = Core;
//     return core;
// };

// module.exports = generate();